package nl.amis.adffaces.library;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.ActionEvent;

import oracle.adf.view.faces.component.core.data.CoreTree;
import oracle.adf.view.faces.model.ChildPropertyTreeModel;
import oracle.adf.view.faces.model.TreeModel;


public class LibraryTreeHandler {

    private CoreTree tree;
    private TreeModel libraryTreeModel;
    private GenericTreeNode selectedNode = null;

    public LibraryTreeHandler() {
        GenericTreeNode library = new GenericTreeNode("Library", "folder");

        GenericTreeNode ben = new GenericTreeNode("Ben Elton", "author");
        String bio = " Already a successful comedian, Ben Elton turned to writing situation comedies during the 1980s and penned BBC classics such as \"The Young Ones\", \"Blackadder\", and during the 1990s \"The Thin Blue Line\".\n" + 
            "    He provided lyrics for Andrew Lloyd Webber Musical, The Beautiful Game, which was nominated for Best Musical at the Laurence Olivier Theatre Awards in 2001 (2000 season).\n" + 
            "    His comedy, Popcorn performed at the Apollo Theatre, was awarded the 1998 Laurence Olivier Theatre Award for Best New Comedy of the 1997 season.\n" + 
            "    Has three children : Bert, Lottie and Fred. Is co-writer of the Queen Musical 'We Will Rock You' with the band itself.\n" + 
            "Birth name: Benjamin Charles Elton. Height: 5' 8\" (1.73 m) \n"; 
        ben.setLongDescription(bio);
        GenericTreeNode book = new GenericTreeNode("Post Mortem", "book");
        ben.getChildren().add(book);
        book = new GenericTreeNode("Blast from the Past", "book");
        ben.getChildren().add(book);
        book = new GenericTreeNode("Stark", "book");
        ben.getChildren().add(book);

        GenericTreeNode rod = new GenericTreeNode("Rod Johnson", "author");
            bio = "Rod Johnson is an enterprise Java architect with extensive experience in the insurance, dot-com, and financial industries. He was the J2EE architect of one of Europe's largest web portals, and he has worked as a consultant on\n" + 
            "a wide range of projects.\n Rod has an arts degree majoring in music and computer science from the University of Sydney. He obtained a Ph.D. in musicology before returning to software development. He has been working with both Java and J2EE since\n" + 
            "their release and is actively involved in the Java Community Process as a member of the JSR-154 (Servlet 2.4) and JDO 2.0 Expert Groups. He is the author of several best-selling books, like \"Expert One-on-One J2EE Design and Development\" (Wrox, 2002) and has contributed to several other books on J2EE since 2000. As founder of the Spring Framework (www.springframework.org), he speaks frequently at leading industry conferences.\n";
        rod.setLongDescription(bio);
        book = new GenericTreeNode("J2EE Design and Development", "book");
        rod.getChildren().add(book);
        book =
            new GenericTreeNode("J2EE without Enterprise Java Bean", "book");
        rod.getChildren().add(book);
        book =
            new GenericTreeNode("Professional Java Development With The Spring Framework",
                                   "book");
        rod.getChildren().add(book);
        
        bio = "Nobody had ever won the Hugo and Nebula awards for best novel two years in a row, until Orson Scott Card received them for Ender's Game and its sequel, Speaker for the Dead, in 1986 and 1987. The third novel in the series, Xenocide, was published in 1991, and the fourth and seemingly final volume, Children of the Mind, was published in August 1996. However, the Ender cycle now includes the new parallel series that began with Ender's Shadow in 1999, followed by Shadow of the Hegemon in 2001, and continued with Shadow Puppets in 2002. Warner Brothers also recently announced that it has made a deal for director Wolfgang Petersen to bring Ender's Game to the big screen. Born in Richland, Washington, Card grew up in California, Arizona, and Utah. He lived in Brazil for two years as an unpaid missionary for the Mormon Church. He received degrees from Brigham Young University (1975) and the University of Utah (1981). He currently lives in Greensboro, North Carolina. He and his wife, Kristine, are the parents of five children: Geoffrey, Emily, Charles, Zina Margaret, and Erin Louisa (named for Chaucer, Bronte and Dickinson, Dickens, Mitchell, and Alcott, respectively).";
        GenericTreeNode orson =
            new GenericTreeNode("Orson Scott Card", "author");
        orson.setLongDescription(bio);
        book = new GenericTreeNode("Ender's Game", "book");
        orson.getChildren().add(book);
        book = new GenericTreeNode("Xenocide", "book");
        orson.getChildren().add(book);
        book = new GenericTreeNode("Shadow Puppets", "book");
        orson.getChildren().add(book);

        library.getChildren().add(ben);
        library.getChildren().add(rod);
        library.getChildren().add(orson);

        // create the list of root nodes:
        List nodes = new ArrayList();
        nodes.add(library);

        libraryTreeModel = new ChildPropertyTreeModel(nodes, "children");
        }

        public TreeModel getLibraryTreeModel() {
        return libraryTreeModel;
        }
    
    public void setTree(CoreTree tree) {
        this.tree=tree;
    }
    
    public CoreTree getTree() {
        return tree;
    }
    
    private GenericTreeNode findNode(String description, Collection<GenericTreeNode> nodes) {
        // iterate through the collection, verify each node and then - if it does not match - verify its child collection
        for (GenericTreeNode node: nodes) {
            if (node.getDescription().equalsIgnoreCase(description)) { return node;}
            GenericTreeNode childNode = findNode(description, node.getChildren());
            if (childNode != null) return childNode;            
        }
        return null;
    }
    
    public void process(ActionEvent event) throws AbortProcessingException {
    FacesContext context = FacesContext.getCurrentInstance();
    javax.servlet.http.HttpServletRequest req = (javax.servlet.http.HttpServletRequest)context.getExternalContext().getRequest();
    String authorName = req.getParameter("authorName");
    // find the selected node in the tree model:    
    Collection roots = (Collection)libraryTreeModel.getWrappedData();
    GenericTreeNode selectedNode = findNode( authorName, roots);
    this.selectedNode = selectedNode;

    // This would seem the more logical code for handling the event and its 
    // associated parameter. Unfortunately, it turns out that this method is
    // invoked for each of the Author nodes in the tree, regardless of the
    // node whose commandlink was actually clicked on; what's more: for each
    // of these invocations, the value of the UIParameter authorName is equal
    // to the node being dealt with. So for three authors, even if we select
    // the second one, this method is called three times with three different
    // values for the UIParameter. However, reading the parameter authorName
    // directly from the HttpServletRequest does get us a single value.
    // I probably make somehow inappropriate use of the ADF Faces tree component...
    /*
    UIComponent component = (UIComponent)event.getSource();
    List children = component.getChildren();
    for (int i=0;i<children.size();i++) {
      if (children.get(i) instanceof UIParameter) {
        UIParameter currentParam = (UIParameter)children.get(i);
        if (currentParam.getName().equals("authorName")) {
            String nodeAuthorName = (String)currentParam.getValue();            
        }            
      }
    }
    }
    // deal with selected Author node
    */
    }

    public void setSelectedNode(GenericTreeNode selectedNode) {
        this.selectedNode = selectedNode;
    }

    public GenericTreeNode getSelectedNode() {
        return selectedNode;
    }
    }
